package edu.psu.geovista.datamining.vis.hd;

/**
 * <p>Title: Interactive Spatial Data Mining for Large and High-Dimensional Data</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: GeoVISTA Center and Department of Geography, Pennsylvania State University</p>
 * @author Diansheng Guo
 * @version 1.0
 */

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;

import edu.psu.geovista.datamining.util.*;
import edu.psu.geovista.ui.event.*;
import edu.psu.geovista.datamining.data.HD.*;
import edu.psu.geovista.datamining.data.*;

/***
 * This visualization tool uses PCP to view dense cells and possible
 * clusters in high-dimensional space.
 * */
public class PCPCanvas
    extends JPanel
    implements MouseListener,
    MouseMotionListener,
    KeyListener,
    /*HighDimensionalClusteringListener,*/
    ColorArrayListener,
    SelectionListener
{
  public boolean debug = false;

  public static final int SCALE_CELL_MAX_MIN = 1000;
  public static final int SCALE_DATA_MAX_MIN = 1100;
  public static final int SCALE_NESTEDMEANS = 1111;
  public static final int SCALE_GLOBAL_MAX_MIN = 2222;

  private static final int SELECTION_NEW = 0;
  private static final int SELECTION_UNION = 1;
  private static final int SELECTION_INTERSECT = 2;

  public static final int DATA_LEVEL_HIGH = 999;
  public static final int DATA_LEVEL_LOW  = 666;

  public static final float maxthickness = 5.0f;
  public static final float minthickness = 2.0f;

  private int scale = this.SCALE_NESTEDMEANS;
  protected String names[] = null;
  protected int datalevel = DATA_LEVEL_HIGH; //defaul is the aggregated data

  //This is where all the data kept
  protected BasicCell[] highlevelCells = null; //high-level cells
  protected BasicCell[] lowlevelCells = null; //low-level cells
  protected int[] highlevelSelectedCells = null;
  protected int highlevelSelectionCount = 0;
  protected int[] highlevelPreSelectedCells = null;
  protected int highlevelPreSelectionCount = 0;
  protected int[] lowlevelSelectedCells = null;
  protected int lowlevelSelectionCount = 0;
  protected int[] lowlevelPreSelectedCells = null;
  protected int lowlevelPreSelectionCount = 0;

  //These are pointers to either highlevel or lowlevel counterparts depending on the current data level
  //They are changed in method setDataLevel()
  private BasicCell[] cells;
  private int selectionCount;
  private int preSelectionCount;
  private int[] selectedCells;
  private int[] preSelectedCells;

  private double xMax;
  private double xMin;

  /**For internal use*/
  private double xScaleRatio;
  private double[] yScaleRatio;

  private int left_margin = 15;
  private int right_margin = 35;
  private int top_margin = 25;
  private int bottom_margin = 35;

  private int left_x;
  private int bottom_y;
  private int top_y;
  private int right_x;

  protected boolean indication = false; // whether to locate a string when mouse is moving.
  protected boolean nestedSelection = false;
  protected boolean unionSelection = false;
  protected boolean showAll = false;

  private float maxlinewidth = 7.0f;
  private float minlinewidth = 0.2f;
  private int maxdensity = Integer.MIN_VALUE;
  private int mindensity = Integer.MAX_VALUE;

  public void setDataLevel(int level){
    if (level == this.DATA_LEVEL_HIGH){
      this.datalevel = this.DATA_LEVEL_HIGH;
      cells = this.highlevelCells;
      selectionCount = this.highlevelSelectionCount;
      selectedCells = this.highlevelSelectedCells;
      preSelectionCount = this.highlevelPreSelectionCount;
      preSelectedCells = this.highlevelPreSelectedCells;
    }else if (level == this.DATA_LEVEL_LOW){
      this.datalevel = this.DATA_LEVEL_LOW;
      cells = this.lowlevelCells;
      selectionCount = this.lowlevelSelectionCount;
      selectedCells = this.lowlevelSelectedCells;
      preSelectionCount = this.lowlevelPreSelectionCount;
      preSelectedCells = this.lowlevelPreSelectedCells;
    }
    this.repaint();
  }

  public void setOrderredDimensions (int[] orderredDims){
    try{
      if (this.orderredDimensions.length != orderredDims.length) {
        throw new Exception("The input have a different number of dimensions. ");
      }
    }catch (Exception ee){
      ee.printStackTrace();
    }
    System.arraycopy(orderredDims, 0, this.orderredDimensions, 0, this.orderredDimensions.length);
    this.repaint();
  }

  /****
   * Set the data.
   */
  private int[] orderredDimensions;
  public void setCells(BasicCell[] cs) {
    if (cs == null)
      return;
    this.highlevelCells = cs;
    this.highlevelSelectedCells = new int[cs.length];
    this.highlevelSelectionCount = 0;
    this.highlevelPreSelectedCells = new int[cs.length];
    this.highlevelPreSelectionCount = 0;

    int count = 0;
    for (int c=0; c<cs.length; c++)
      count = count + cs[c].getCountOfInstances();
    this.lowlevelSelectedCells = new int[count];
    this.lowlevelSelectionCount = 0;
    this.lowlevelPreSelectedCells = new int[count];
    this.lowlevelPreSelectionCount = 0;

    ///the default order is the original order in the data
    this.orderredDimensions = new int[cs[0].getDimensionality()];
    for (int i=0; i< orderredDimensions.length; i++)
      orderredDimensions[i] = i;

    //Initialize
    this.init();

    //Calculate nested means for each dimension,
    //Due to possible too many identical values, sometime the 9 nested mean values
    //, including max and min, can have two or more same values.
    //Procedure below is to remove such duplicate values, therefore the number of value ticks
    //for different dimension may vary.
    NestedMean[] nm = this.calculateNestedMeans();
    int level = 3;
    int len = nm[0].getMeans(level).length;
    this.nestedMeans = new double[nm.length][]; //[len + 2]
    double[] temp;
    double[] validintervals = new double[len + 2];
    for (int d = 0; d < nm.length; d++) {
      int countintervals = 0;
      temp = nm[d].getMeans(level);
      validintervals[countintervals] = this.yDataMin[d];
      for (int i = 0; i < temp.length; i++) {
        if (Double.compare(temp[i], validintervals[countintervals]) == 0)
          continue;
        countintervals++;
        validintervals[countintervals] = temp[i];
      }
      if (Double.compare(this.yDataMax[d], validintervals[countintervals]) > 0) {
        countintervals++;
        validintervals[countintervals] = this.yDataMax[d];
      }
      countintervals++;

      this.nestedMeans[d] = new double[countintervals];
      System.arraycopy(validintervals, 0, this.nestedMeans[d], 0,
                       countintervals);
    }

    //Calculate synthetic values for each cell
    int numIntervals = this.nestedMeans[0].length - 1;

    int[] dims = this.highlevelCells[0].getDimensions();
    int position = -1;
    int v;

    for (int i = 0; i < this.highlevelCells.length; i++) {
      //double[] values = new double[dims.length];
      double value = -1;
      double synVal;
      double span = 100;
      double middle = 50;

      for (int d = 0; d < dims.length; d++) {
        numIntervals = this.nestedMeans[d].length - 1;
        this.ySyntheticMin[d] = -0.5;
        this.ySyntheticMax[d] = numIntervals - 0.5;

        value = this.highlevelCells[i].getAverage(d);
        for (v = 1; v < numIntervals; v++) {
          if (value < this.nestedMeans[d][v])
            break;
        }

        position = v - 1;
        span = this.nestedMeans[d][position + 1] - this.nestedMeans[d][position];
        middle = (this.nestedMeans[d][position + 1] +
                  this.nestedMeans[d][position]) / 2;
        if (Double.isNaN(span) || Double.isNaN(middle) ||
            Double.compare(span, 0) == 0)
          synVal = position;
        else
          synVal = ( (value - middle) / span) + position;
        this.highlevelCells[i].setSyntheticValue(d, synVal);
      }
    }

    //for all lowlevel cells
    for (int i = 0; i < this.lowlevelCells.length; i++) {
      double synVal;
      double value = -1;
      double span = 100;
      double middle = 50;

      for (int d = 0; d < dims.length; d++) {
        numIntervals = this.nestedMeans[d].length - 1;
        value = this.lowlevelCells[i].getAverage(d);
        for (v = 1; v < numIntervals; v++) {
          if (value < this.nestedMeans[d][v])
            break;
        }
        position = v - 1;
        span = this.nestedMeans[d][position + 1] - this.nestedMeans[d][position];
        middle = (this.nestedMeans[d][position + 1] +
                  this.nestedMeans[d][position]) / 2;
        if (Double.isNaN(span) || Double.isNaN(middle) ||
            Double.compare(span, 0) == 0)
          synVal = position;
        else
          synVal = ( (value - middle) / span) + position;
        this.lowlevelCells[i].setSyntheticValue(d, synVal);
      }
    }

    this.validate();
    this.setDataLevel(this.datalevel);
  }

  /***
   * Set the scaling method: nested means, or cell max-min, or data max-min
   */
  public void setAxisScalingMethod(int sc) {
    if (this.scale == sc)
      return;
    if (sc == this.SCALE_NESTEDMEANS)
      this.scale = this.SCALE_NESTEDMEANS;
    else if (sc == this.SCALE_CELL_MAX_MIN)
      this.scale = this.SCALE_CELL_MAX_MIN;
    else if (sc == this.SCALE_DATA_MAX_MIN)
      this.scale = this.SCALE_DATA_MAX_MIN;
    else if (sc == this.SCALE_GLOBAL_MAX_MIN)
      this.scale = this.SCALE_GLOBAL_MAX_MIN;
    this.repaint();
  }

  public int getAxisScalingMethod(){
    return this.scale;
  }

  /***
   * Constructor
   * */
  public PCPCanvas() {
    addMouseListener(this);
    addMouseMotionListener(this);
    this.setBackground(ColorSetting.background);
    this.setLayout(null);
    this.repaint();
  }

  protected void setNestedSelection(boolean b) {
    this.nestedSelection = b;
  }

  protected void setUnionSelection(boolean b){
    this.unionSelection = b;
  }

  protected void setIndication(boolean ind) {
    this.indication = ind;
  }

  /***
   * This method has two tasks:
   * (1) calcualted the nested means for all dimensions
   * (2) costruct an array of low-level basic cells, each of which holds a single data item.
   */
  private NestedMean[] calculateNestedMeans(){
  //calculate nested means for each dimension
    int[] dims = this.highlevelCells[0].getDimensions();
    NestedMean[] nestedmeans = new NestedMean[dims.length];

    int count = 0;
    for (int i = 0; i < this.highlevelCells.length; i++) {
      if (this.highlevelCells[i] == null)
        continue;
      count = count + highlevelCells[i].getCountOfInstances();
    }

    this.lowlevelCells = new BasicCell[count];
    BasicInstanceInterface[] instances;
    int id;
    for (int i = 0; i < this.highlevelCells.length; i++) {
      if(this.highlevelCells[i] == null)
        continue;
      instances = highlevelCells[i].getInstances();
      for (int k = 0; k < highlevelCells[i].getCountOfInstances(); k++) {
        id = instances[k].getId();
        lowlevelCells[id] = new BasicCell(id, dims, null, true);
        lowlevelCells[id].addInstance(instances[k]);
        lowlevelCells[id].color = highlevelCells[i].color;
      }
    }

    double values[] = new double[count];
    int dimension;
    for (int d=0; d< dims.length; d++){
      dimension = dims[d];
      count = 0;
      for (int i = 0; i < this.highlevelCells.length; i++) {
        if(this.highlevelCells[i] == null)
          continue;
        instances = highlevelCells[i].getInstances();
        for (int k = 0; k < highlevelCells[i].getCountOfInstances(); k++) {
          values[count] = instances[k].getOriginalElementValue(dimension);
          count++;
        }
      }
      Arrays.sort(values, 0, count);
      nestedmeans[d] = new NestedMean(values, 0, count - 1, 0, 4);
    }

    return nestedmeans;
  }

  public BasicCell[] getHighLevelCellsWithSyntheticValues(){
    return this.highlevelCells;
  }

  private double[][] nestedMeans = null;
  private void init() {
    int d = this.highlevelCells[0].getDimensionality();
    this.xMin = 0;
    this.xMax = d - 1;
    yCellMax = new double[d];
    yCellMin = new double[d];
    yDataMax = new double[d];
    yDataMin = new double[d];
    ySyntheticMax = new double[d];
    ySyntheticMin = new double[d];

    for (int i = 0; i < d; i++) {
      yCellMax[i] = Integer.MIN_VALUE;
      yCellMin[i] = Integer.MAX_VALUE;
      yDataMax[i] = Integer.MIN_VALUE;
      yDataMin[i] = Integer.MAX_VALUE;
    }

    maxdensity = Integer.MIN_VALUE;
    mindensity = Integer.MAX_VALUE;
    int den;
    BasicCell cell;
    for (int c = 0; c < this.highlevelCells.length; c++) {
      cell = highlevelCells[c];
      den = cell.getCountOfInstances();
      if (den > maxdensity)
        maxdensity = den;
      if (den < mindensity)
        mindensity = den;
      for (int i = 0; i < d; i++) {
        if (cell.getAverage(i) > yCellMax[i]) yCellMax[i] = cell.getAverage(i);
        if (cell.getAverage(i) < yCellMin[i]) yCellMin[i] = cell.getAverage(i);
        if (cell.getDataMax(i) > yDataMax[i]) yDataMax[i] = cell.getDataMax(i);
        if (cell.getDataMin(i) < yDataMin[i]) yDataMin[i] = cell.getDataMin(i);
      }
    }

    //if the maxdensity < (maxlinewidth - minlinewidth), using the actual value range
    if (maxdensity < this.maxlinewidth/2){
      this.maxlinewidth = this.maxthickness - (this.maxlinewidth-maxdensity-2)/2;
      this.minlinewidth = this.minthickness + (this.maxlinewidth-maxdensity-2)/2;
    }else{
      this.maxlinewidth = this.maxthickness;
      this.minlinewidth = this.minthickness;
    }


    int len = this.highlevelCells.length;
    int count = 0;
    for (int i = 0; i < this.highlevelCells.length; i++) {
      if (this.highlevelCells[i] == null)
        continue;
      count = count + highlevelCells[i].getCountOfInstances();
    }
    Color colors[] = new Color[count]; //because maxid starts from 0
    for (int c = 0; c < colors.length; c++)
      colors[c] = ColorSetting.disabled; //default color for objects having null values is BLACK.

    Color c;
    for (int i = 0; i < len; i++) {
      int[] ids = this.highlevelCells[i].getInstanceIndice();
      c = this.highlevelCells[i].color;
      for (int tt = 0; tt < ids.length; tt++)
        colors[ids[tt]] = c;
    }
    ColorArrayEvent ee = new ColorArrayEvent(this, colors);

    fireColorArrayChanged(ee);
  }

  public void setAttributeNames(String names[]) {
    this.names = names;
  }

  private void configureScaleRatios() {
    int d = this.highlevelCells[0].getDimensionality();
    int w = this.getWidth();
    int h = this.getHeight();

    if (this.drawingBuff == null ||
        w != this.drawingBuff.getWidth(this) ||
        h != this.drawingBuff.getHeight(this)) {
      this.drawingBuff = this.createImage(w, h);
    }
    xScaleRatio = (w - 6 - this.left_margin - this.right_margin) /
        (this.xMax - this.xMin);
    if (yScaleRatio == null || yScaleRatio.length != d)
      yScaleRatio = new double[d];
    for (int i = 0; i < d; i++)
      yScaleRatio[i] = (h - 6 - this.top_margin - this.bottom_margin) /
          (this.getYMax(i) - this.getYMin(i));

    left_x = (int) (this.left_margin);
    bottom_y = (int) (h - 6 - this.bottom_margin);
    right_x = (int) (w - 6 - this.right_margin);
    top_y = (int) (this.top_margin);
  }

  private Image drawingBuff = null;
  /***
   * This is the overall method that draws everything. It takes several steps
   * to configure the visualization
   * (1) According to the dimensionality (d) and the width of the current window
   * to decide if we need the horizontal scroll bar.
   * (2) Using the width and height of the window to configure the zoomin/zoomout ratio
   * for x and y.
   */
  public void paintComponent(Graphics g) {
    super.paintComponent(g); //paint background
    //if (this.highlevelCells == null)
    if (this.cells == null)
      return;
    //Decide if a horizontal scroll bar is needed.
    //Also calculate scale ratios
    this.configureScaleRatios();
    Graphics2D gbuff2d = (Graphics2D)( this.drawingBuff.getGraphics());
    gbuff2d.setColor(this.getBackground());
    gbuff2d.fillRect(0, 0, this.getWidth(), this.getHeight());
    this.drawDimensionLines(gbuff2d, xScaleRatio, yScaleRatio);
    this.drawValues(gbuff2d, xScaleRatio, yScaleRatio);
    this.drawValueTicks(gbuff2d, xScaleRatio, yScaleRatio);
    Graphics2D g2d = (Graphics2D) g;
    g2d.drawImage(this.drawingBuff, 0, 0, this.getWidth(), this.getHeight(), this);
  }


  /***
   *
   */
  private void drawDimensionLines(Graphics2D g, double xratio, double yratio[]) {
    Color temp = g.getColor();
    g.setColor(ColorSetting.PCP_AXIS_COLOR);
    int x;
    int dims[] = this.highlevelCells[0].getDimensions();
    for (int i = 0; i <= this.xMax; i++) {
      x = (int) (i * xratio) + this.left_x;
      g.fillRect(x - 1, top_y, 3, bottom_y - top_y);
    }
    g.setColor(ColorSetting.PCP_AXIX_NAME_COLOR);
    int d;
    for (int i = 0; i <= this.xMax; i++) {
      x = (int) (i * xratio) + 1 - (int) (i * (15.0 / this.xMax));
      d = this.orderredDimensions[i];
      g.drawString(names[d], x, bottom_y + this.bottom_margin * 2 / 3);
    }
    g.setColor(temp);
  }

  private void drawValueTicks(Graphics2D g, double xratio, double yratio[]){
    Color temp = g.getColor();
    g.setColor(ColorSetting.PCP_AXIX_VALUE_COLOR);
    int vlabelx;
    double values[];
    int len;
    for (int d = 0; d <= this.xMax; d++) {
      vlabelx = (int) (d * xratio) + 1;
      values = this.getYValueTicks(this.orderredDimensions[d]);
      len = values.length;
      for (int m = 0; m < values.length; m++){
        g.drawString("" + values[m],
                     vlabelx,
                     (int) (bottom_y - m * (bottom_y - top_y) / (len-1) + 0.5));
      }
    }
    g.setColor(temp);
  }




  private java.awt.geom.GeneralPath selectedPath = null;
   private void drawCell(Graphics2D g,
                        double xratio,
                        double[] yratio,
                        float widthratio,
                        int cellid,
                        BasicCell cell,
                        Color color,
                        int dim_size,
                        boolean selected,
                        double portion) {
    /*If the portion is smaller than 5%, ignore this cell. */
    if (portion < 0.005)
      return;
    int x;
    int y;
    int density;
    float linewidth; //The width of each string is proportional to its density.
    density = (cell.getCountOfInstances()); // * portion + 0.5);

    if ((this.maxdensity - this.mindensity) < (this.maxlinewidth - this.minlinewidth)){
      linewidth = (density - mindensity); // + minlinewidth;
    }else
      linewidth = (density - mindensity) * widthratio; // + minlinewidth;
    linewidth = linewidth * (float)portion + minlinewidth;

    if (linewidth < 0)
      linewidth = 0;
    g.setStroke(new BasicStroke(linewidth, BasicStroke.CAP_ROUND,
                                BasicStroke.JOIN_ROUND));
    g.setColor(color);
    int xs[]= new int[dim_size];
    int ys[]= new int[dim_size];
    double value;
    int actualD;
    for (int d = 0; d < dim_size; d++) {
      x = (int) (d * xratio) + this.left_x;
      actualD = this.orderredDimensions[d];
      value = this.getCellValue(cellid, cell, actualD);
      y = bottom_y - (int) ( (value - this.getYMin(actualD)) * yScaleRatio[actualD] + 0.5);
      xs[d] = x;
      ys[d] = y;
    }
    g.drawPolyline(xs, ys, dim_size);
  }

  /***
   *
   * */
  private void drawValues(Graphics2D g, double xratio, double[] yratio) {
    //if (highlevelCells == null || highlevelCells.length < 1)
    if (cells == null || cells.length < 1)
      return;

    float widthratio = 1.0f; //default is 1.0, especially when max == min
    if (maxdensity - mindensity > 0.001f) //not equal to each other
      widthratio = (maxlinewidth - minlinewidth) / (maxdensity - mindensity);
    Color temp = g.getColor();
    int dim_size = this.highlevelCells[0].getDimensionality();
    if (selectionCount > 0 || (selectionCount <= 0 && this.portions != null)) {
      System.out.println("draw all currentLevelCells.");
      if (this.portions == null)
        for (int i = 0; i < cells.length; i++) {
          this.drawCell(g,
                        xratio,
                        yratio,
                        widthratio,
                        i,
                        cells[i],
                        ColorSetting.disabled,
                        dim_size,
                        false,
                        1);
        }
      //draw selected values
      double portion = 1;
      int cid;
      for (int i = 0; i < selectionCount; i++) {
        if (this.portions != null && this.datalevel == this.DATA_LEVEL_HIGH)
          portion = this.portions[i];
        else
          portion = 1;
        cid = selectedCells[i];
        this.drawCell(g,
                      xratio,
                      yratio,
                      widthratio,
                      cid,
                      cells[cid],
                      cells[selectedCells[i]].color,
                      dim_size,
                      true,
                      portion);
      }
    }
    else {
      //draw disabled currentLevelCells.
      for (int i = 0; i < cells.length; i++) {
        if (cells[i].isDense())
          continue;
        this.drawCell(g,
                      xratio,
                      yratio,
                      widthratio,
                      i,
                      cells[i],
                      ColorSetting.disabled,
                      dim_size,
                      false,
                      1);
      }
      //draw dense currentLevelCells
      for (int i = 0; i < cells.length; i++) {
        if (!cells[i].isDense())
          continue;
        this.drawCell(g,
                      xratio,
                      yratio,
                      widthratio,
                      i,
                      cells[i],
                      cells[i].color,
                      dim_size,
                      false,
                      1);
      }
    }
    g.setColor(temp);
  }


  /***
    * This method only draw those selected cells. Thus it is fast in reponding
    * user's interactions.
    */
   public void drawSelectedCells() {
     if (highlevelCells == null)
       return;

     if (selectionCount <= 0) {
       selectedPath = null;
       this.repaint();
       return;
     }

     Graphics2D g2d = (Graphics2D) (this.getGraphics());
     ////redraw the preSelected points.
     if (preSelectionCount > 0) {
       this.drawPreSelectedValuesOnly(g2d, xScaleRatio, yScaleRatio);
     }
     else {
       this.repaint();
       return;
     }
     this.drawSelectedValuesOnly(g2d, xScaleRatio, yScaleRatio);
     this.drawValueTicks(g2d, xScaleRatio, yScaleRatio);
   }

   /***
     *
     * */
    private void drawSelectedValuesOnly(Graphics2D g, double xratio, double yratio[]) {
      if (highlevelCells == null || highlevelCells.length < 1)
        return;

      float widthratio = (maxlinewidth - minlinewidth)/(maxdensity - mindensity);
      int dim_size = cells[0].getDimensionality();
      BasicCell cell;
      double portion = 1;
      for (int i = 0; i < selectionCount; i++) {
        cell = cells[selectedCells[i]];
        if (this.portions != null)
          portion = this.portions[i];
        else
          portion = 1;
        this.drawCell(g,
                      xratio,
                      yratio,
                      widthratio,
                      selectedCells[i],
                      cells[selectedCells[i]],
                      cell.color,
                      dim_size,
                      true,
                      portion);
      }
    }

    /***
      * This is the main method that draw the density plot.
      * */
     private void drawPreSelectedValuesOnly(Graphics2D g, double xratio, double yratio[]) {
       if (highlevelCells == null || highlevelCells.length < 1)
         return;

       float widthratio = (maxlinewidth - minlinewidth) / (maxdensity - mindensity);
       int dim_size = this.highlevelCells[0].getDimensionality();
       int cid = -1;
       for (int i = 0; i < preSelectionCount; i++) {
         cid = preSelectedCells[i];

         this.drawCell(g,
                       xratio,
                       yratio,
                       widthratio,
                       cid,
                       cells[cid],
                       this.getBackground(),
                       dim_size,
                       false,
                       1);
         this.drawCell(g,
                       xratio,
                       yratio,
                       widthratio,
                       cid,
                       cells[cid],
                       ColorSetting.disabled,
                       dim_size,
                       false,
                       1);
       }
     }


  /***
   * Interface methods for mouse events
   *  * **/
  public void mouseClicked(MouseEvent e) {
  }

  private int mousePressedX = 0;
  private int mousePressedY = 0;
  public void mousePressed(MouseEvent e) {
    mousePressedX = e.getX();
    mousePressedY = e.getY();
  }

  private int mouseReleasedX = 0;
  private int mouseReleasedY = 0;
  //private int[] selectionIdx = null;
  //private int   selectionCount = 0;
  private boolean selectionAll = false;

  /**
   * Input: two mouse locations (begining of dragging and ending of dragging)
   *
   * return: the number of selected instances.
   *
   * */
  private void doSelection(int x1, int y1, int x2, int y2, int selectionType) {
    if (this.highlevelCells == null)
      return;

    int left_dx = 0;
    int right_dx = 0;
    int dim = -1;
    int dim2 = -1;
    int dim_size = cells[0].getDimensionality();
    for (int d = 0; d < dim_size - 1; d++) {
      left_dx = (int) (d * xScaleRatio) + this.left_x;
      right_dx = (int) ( (d + 1) * xScaleRatio) + this.left_x;
      if (x1 > left_dx && x1 < right_dx){
        if (x2 > left_dx && x2 < right_dx) {
          dim = d;
          break;
        }
        else {
          JOptionPane.showMessageDialog(this,
                                        "When selecting by dragging the mouse, please do not cross dimension lines.");
          return;
        }
      }
    }
    if (dim == -1)
      return;
    dim2 = this.orderredDimensions[dim+1];
    dim = this.orderredDimensions[dim];

    int left_dy, right_dy;
    double v1 = -1, v2 = -1;

    if (selectionType == this.SELECTION_NEW) {
      selectionCount = 0;
      for (int i = 0; i < cells.length; i++) {
        left_dy = bottom_y -
            (int) ( (this.getCellValue(i, cells[i], dim) - this.getYMin(dim)) *
                   yScaleRatio[dim] + 0.5);
        right_dy = bottom_y -
            (int) ( (this.getCellValue(i, cells[i], dim2) - this.getYMin(dim2)) *
                   yScaleRatio[dim2] + 0.5);
        v1 = (double) (y1 - right_dy) / (double) (x1 - right_dx) -
            (double) (left_dy - right_dy) / (double) (left_dx - right_dx);
        v2 = (double) (y2 - right_dy) / (double) (x2 - right_dx) -
            (double) (left_dy - right_dy) / (double) (left_dx - right_dx);

        if (v1 * v2 <= 0) { //no nested selection...on different sides of the string, select it.
          selectedCells[selectionCount] = i;
          selectionCount++;
        }
      }
    }
    else if (selectionType == this.SELECTION_INTERSECT){ //nested selection
      selectionCount = 0;
      for (int i = 0; i < preSelectionCount; i++) {
        left_dy = bottom_y -
            (int) ( (this.getCellValue(preSelectedCells[i],
                                       cells[preSelectedCells[i]], dim) -
                     this.getYMin(dim)) *
                   yScaleRatio[dim] + 0.5);
        right_dy = bottom_y -
            (int) ( (this.getCellValue(preSelectedCells[i],
                                       cells[preSelectedCells[i]], dim2) -
                     this.getYMin(dim2)) *
                   yScaleRatio[dim2] + 0.5);
        v1 = (double) (y1 - right_dy) / (double) (x1 - right_dx) -
            (double) (left_dy - right_dy) / (double) (left_dx - right_dx);
        v2 = (double) (y2 - right_dy) / (double) (x2 - right_dx) -
            (double) (left_dy - right_dy) / (double) (left_dx - right_dx);
        if (v1 * v2 <= 0) {
          selectedCells[selectionCount] = preSelectedCells[i];
          selectionCount++;
        }
      }
      if (selectionCount == 0)
        JOptionPane.showMessageDialog(this, "Nothing selected--\nI think you should set mode to 'Normal Selection' rather than 'Interset Sel.'");
    }
    else if (selectionType == this.SELECTION_UNION){ //only difference here is that we don't reset currentSelectionCount.
      for (int i = 0; i < cells.length; i++) {
        left_dy = bottom_y -
            (int) ( (this.getCellValue(i, cells[i], dim) - this.getYMin(dim)) *
                   yScaleRatio[dim] + 0.5);
        right_dy = bottom_y -
            (int) ( (this.getCellValue(i, cells[i], dim2) - this.getYMin(dim2)) *
                   yScaleRatio[dim2] + 0.5);
        v1 = (double) (y1 - right_dy) / (double) (x1 - right_dx) -
            (double) (left_dy - right_dy) / (double) (left_dx - right_dx);
        v2 = (double) (y2 - right_dy) / (double) (x2 - right_dx) -
            (double) (left_dy - right_dy) / (double) (left_dx - right_dx);
        if (v1 * v2 <= 0) {
          selectedCells[selectionCount] = i;
          selectionCount++;
        }
      }
      System.out.println("union selection: " + selectionCount);
    }
    //Translate selection made at one level to the other level.
    this.translateSelection();
    return;
  }

  public void mouseReleased(MouseEvent e) {
    this.mouseDraggedX = -1;
    this.mouseDraggedY = -1;
    this.mouseDraggedPreX = -1;
    this.mouseDraggedPreY = -1;
    if (this.highlevelCells == null)
      return;

    mouseReleasedX = e.getX();
    mouseReleasedY = e.getY();
    //do selection.
    if (mouseDragged) {
        System.arraycopy(selectedCells, 0, preSelectedCells, 0, selectionCount);
        preSelectionCount = selectionCount;
      //Inside following calls to the doSelection method, no mattter which level is the current selectin made
      //it will be translated to the other level.
      if (e.isShiftDown() || this.unionSelection)
        this.doSelection(mousePressedX, mousePressedY, mouseReleasedX,
                         mouseReleasedY, this.SELECTION_UNION);
      else if (this.nestedSelection)
        this.doSelection(mousePressedX, mousePressedY, mouseReleasedX,
                         mouseReleasedY, this.SELECTION_INTERSECT);
      else {
        this.doSelection(mousePressedX, mousePressedY, mouseReleasedX,
                         mouseReleasedY, this.SELECTION_NEW);
      }
      //Translate the selection at one level to the other level
      this.translateSelection();

      int[] selected = new int[this.lowlevelSelectionCount];
      for (int i=0; i< this.lowlevelSelectionCount; i++){
        selected[i] = this.lowlevelSelectedCells[i];
      }
      //When coordinating with other component, we use ID to identify the cell,
      //not its position in the cells array...because different components may hold
      //a different subset of those cells.
      int highLevelSel[] = new int[this.highlevelSelectionCount];
      for (int i=0; i< this.highlevelSelectionCount; i++){
        highLevelSel[i] = this.highlevelSelectedCells[i];
      }
      if (this.datalevel == this.DATA_LEVEL_HIGH)
        this.fireSelectionChanged(new SelectionEvent(this, selected, highLevelSel, null));
      else if (this.datalevel == this.DATA_LEVEL_LOW)
        this.fireSelectionChanged(new SelectionEvent(this, selected, highLevelSel, this.portions));
    }

    mouseDragged = false;
  }

  public void mouseEntered(MouseEvent e) {
//    this.requestFocus();
  }

  public void mouseExited(MouseEvent e) {
  }

  private boolean mouseDragged = false;
  private int mouseDraggedX = -1;
  private int mouseDraggedY = -1;
  private int mouseDraggedPreX = -1;
  private int mouseDraggedPreY = -1;

  public void mouseDragged(MouseEvent e) {
    mouseDragged = true;
    mouseDraggedPreX = mouseDraggedX;
    mouseDraggedPreY = mouseDraggedY;
    mouseDraggedX = e.getX();
    mouseDraggedY = e.getY();

    Graphics g = this.getGraphics();
    g.setXORMode(Color.white);
    if (this.mouseDraggedPreX != -1)
      g.drawLine(mousePressedX, mousePressedY, mouseDraggedPreX, mouseDraggedPreY);
    g.drawLine(mousePressedX, mousePressedY, mouseDraggedX, mouseDraggedY);
    g.setPaintMode();
  }

//  int preSelection = -1;
//  int currentSelection = -2;
  public void mouseMoved(MouseEvent e) {
//    if (indication) {
//      //first get between which two dimensions the mouse is
//      currentSelection = this.getInstanceByCoordinates(e.getX(), e.getY());
//      if (currentSelection == -1 && this.selectionCount > 0) { //nothing selected
//      }
//      else if (currentSelection != preSelection) {
//        System.arraycopy(this.selectedCells, 0, this.preSelectedCells, 0, this.selectionCount);
//        this.preSelectionCount = this.selectionCount;
//        this.selectedCells[0] = currentSelection;
//        this.selectionCount = 1;
//        //this.repaint();
//        this.drawSelectedCells();
//
//        int[] selected = this.cells[this.selectedCells[0]].getInstanceIndice();
//        //int highLevelSel[] = new int[]{(int)this.cells[currentSelection].getId()};
//        int highLevelSel[] = new int[]{currentSelection};
//        this.adjustedvalues = false;
//
//        this.fireSelectionChanged(new SelectionEvent(this, selected, highLevelSel));
        ///////fireIndicationChanged(new VisEventEvent(this, Color.pink));
//      }
//    }
  }


  /***
   * First, update the numbers  (e.g., highlevel or lowlevel selection / preselection count) with
   * the neutral numbers (e.g., selectionCount / preSelectionCount)
   * Second, translate selection at one level to the other level.
   */
  private void translateSelection(){
    if (this.datalevel == this.DATA_LEVEL_HIGH) {
      this.highlevelPreSelectionCount = this.preSelectionCount;
      this.highlevelSelectionCount = this.selectionCount;
      this.translateSelectionHighToLow();
    }
    else if (this.datalevel == this.DATA_LEVEL_LOW) {
      this.lowlevelPreSelectionCount = this.preSelectionCount;
      this.lowlevelSelectionCount = this.selectionCount;
      this.translateSelectionLowToHigh();
    }
  }

  private void translateSelectionHighToLow(){
    int count = 0;
    int id;
    BasicCell cell;
    BasicInstanceInterface[] instances;
    for (int i = 0; i < this.highlevelSelectionCount; i++) {
      if (this.highlevelCells[i] == null)
        continue;
      cell = highlevelCells[this.highlevelSelectedCells[i]];
      instances = cell.getInstances();
      for (int data=0; data < cell.getCountOfInstances(); data++){
        id = instances[data].getId();
        this.lowlevelSelectedCells[count] = id;
        count ++;
      }
    }
    this.lowlevelSelectionCount = count;
    this.lowlevelPreSelectionCount = 0;
    this.adjustedvalues = false;
    this.portions = null;

    this.setDataLevel(this.datalevel);
  }

  private void translateSelectionLowToHigh(){
    /**If a cell is partially selected, its mean values will be re-calcualted
     * according to its selected items. */
    int cellscount = this.highlevelCells.length;
    int dimensionality = this.highlevelCells[0].getDimensionality();
    if (newmeanvalues == null ||
        newmeanvalues.length != cellscount ||
        newmeanvalues[0].length != dimensionality)
      this.newmeanvalues = new double[cellscount][dimensionality];

    //Initialize the new mean vectors
    portions = new double[cellscount];
    int totalDataItems = 0;
    for (int i = 0; i < this.newmeanvalues.length; i++){
      portions[i] = 0;
      totalDataItems = totalDataItems + this.highlevelCells[i].getCountOfInstances();
      for (int k = 0; k < this.newmeanvalues[0].length; k++)
        this.newmeanvalues[i][k] = 0;
    }

    /**This map is used to quickly check whether a data item is selected or not. */
    int selectionmap[] = new int[totalDataItems];
    for (int i=0; i< this.lowlevelSelectionCount; i++)
      selectionmap[this.lowlevelSelectedCells[i]] = 1;

    /**For each data items in each cell, if it's selected, count it in the portion array.
     * Its values are also added to the newmeanvalues array.
     * */
    BasicInstanceInterface[] insts = null;
    int count =0;
    int dims = this.highlevelCells[0].getDimensionality();
    int[] selDims = this.highlevelCells[0].getDimensions();
    for (int somcell=0; somcell < this.highlevelCells.length; somcell++){
      insts = this.highlevelCells[somcell].getInstances();
      count = this.highlevelCells[somcell].getCountOfInstances();
      for (int inst=0; inst < count; inst++){
        int id = insts[inst].getId();
        //if it's selected...!
        if (selectionmap[id] == 1){
          portions[somcell] = portions[somcell] + 1;
          for (int d=0; d< dims; d++)
            newmeanvalues[somcell][d] = newmeanvalues[somcell][d] + insts[inst].getOriginalElementValue(selDims[d]);
        }
      }
      if (portions[somcell] < 1){
        for (int d = 0; d < dims; d++) {
          newmeanvalues[somcell][d] = this.highlevelCells[somcell].getAverage(d);
        }
      }else
        for (int d = 0; d < dims; d++) {
          newmeanvalues[somcell][d] = newmeanvalues[somcell][d]/ portions[somcell];
        }
      portions[somcell] = portions[somcell] / count;
    }

    count =0;
    for (int somcell = 0; somcell < this.highlevelCells.length; somcell++)
      if (portions[somcell] > 0)
        count++;
    this.highlevelSelectionCount =  count;
    this.highlevelPreSelectionCount = 0;

    if (this.highlevelSelectionCount ==0){
      this.adjustedvalues = false;
      this.portions = null;
      this.setDataLevel(this.datalevel);
      return;
    }
    count = 0;
    for (int somcell = 0; somcell < this.highlevelCells.length; somcell++)
      if (portions[somcell] > 0) {
        this.highlevelSelectedCells[count] = somcell;
        this.portions[count] = portions[somcell];
        count++;
      }
    ////Adjust the vector values for a cell is only part of its data items are selected.
    this.adjustSyntheticValues();
    this.adjustedvalues = true;
    //Will point the pointers to new changes and repaint.
    this.setDataLevel(this.datalevel);
  }


  private double getCellValue(int cid, BasicCell c, int dim){
    if (this.datalevel == this.DATA_LEVEL_HIGH){
      if (!this.adjustedvalues){
        if (this.scale == this.SCALE_NESTEDMEANS) {
          return c.getSyntheticValue(dim);
        }
        else
          return c.getAverage(dim);
      }else {//get adjust values for partial selection in each cell
        if (this.scale == this.SCALE_NESTEDMEANS) {
          return this.newmeanvalues[cid][dim];
        }
        else
          return this.originalnewmeans[cid][dim];
      }
    }else {
        if (this.scale == this.SCALE_NESTEDMEANS)
          return c.getSyntheticValue(dim);
        else
          return c.getAverage(dim);
    }
  }


  private double getYMin(int dim){
    if (this.scale == this.SCALE_NESTEDMEANS)
      return this.ySyntheticMin[dim];
    else if (this.scale == this.SCALE_CELL_MAX_MIN)
      return this.yCellMin[dim];
    else if (this.scale == this.SCALE_GLOBAL_MAX_MIN)
      return MathUtil.getMax(this.yDataMin);
    else
      return this.yDataMin[dim];
  }

  private double getYMax(int dim){
    if (this.scale == this.SCALE_NESTEDMEANS)
      return this.ySyntheticMax[dim];
    else if (this.scale == this.SCALE_CELL_MAX_MIN)
      return this.yCellMax[dim];
    else if (this.scale == this.SCALE_GLOBAL_MAX_MIN)
      return MathUtil.getMax(this.yDataMax);
    else
      return this.yDataMax[dim];
  }

  private double[] getYValueTicks(int d){
    double values[] = new double[this.nestedMeans[d].length];
    if (this.scale == this.SCALE_NESTEDMEANS){
        for (int m = 0; m < values.length ; m++){
          values[m] = ((int)((this.nestedMeans[d][m])*100))/100.0;
        }
        return values;
    }
    else{
      values[0] = ( (int) ( (this.getYMin(d)) * 100)) / 100.0;
      values[values.length - 1] = ( (int) ( (this.getYMax(d)) * 100)) / 100.0;
      double interval = (values[values.length - 1] - values[0]) / (values.length -1);
      for (int m = 1; m < values.length-1; m++) {
        values[m] = ( (int) ( (values[0] + interval * m) * 100)) / 100.0;
      }
      return values;
    }
  }

  public void keyTyped(KeyEvent e) {
  }

  public void keyPressed(KeyEvent e) {
  }

  public void keyReleased(KeyEvent e) {
  }

  public void colorArrayChanged(ColorArrayEvent e) {
    if (this.lowlevelCells != null){
      BasicInstanceInterface[] instances;
      for (int i = 0; i < this.highlevelCells.length; i++) {
        if (this.highlevelCells[i] == null)
          continue;
        instances = highlevelCells[i].getInstances();
        for (int k = 0; k < highlevelCells[i].getCountOfInstances(); k++) {
          lowlevelCells[instances[k].getId()].color = highlevelCells[i].color;
        }
      }
    }
    this.repaint();
  }

  /***
   * Responding to multi-item selection.
   */
  private double[] portions = null;
  private double[][] newmeanvalues = null;
  private boolean adjustedvalues = false;

  public void selectionChanged(SelectionEvent e) {
    if (this.highlevelCells == null || this.highlevelCells.length == 0)
      return;

    //If any, this array contains all data level (as oppose to aggregation level)
    // ids that are currently selected.
    int[] datalevelsel = e.getSelection();
    //If any, this array contains all aggregation level ids that are currently selected.
    int[] grouplevelsel = e.getHigherLevelSelection();
    //Initialize portions array and the newmeanvalues array.
    portions = null;
    this.adjustedvalues = false;

    /** If no selection at all, return. */
    if ((datalevelsel == null || datalevelsel.length == 0)&& (grouplevelsel == null || grouplevelsel.length == 0)){
      this.highlevelSelectionCount = 0;
      this.lowlevelSelectionCount = 0;
      this.repaint();
      return;
    }else{
      if (grouplevelsel != null && grouplevelsel.length > 0 && (datalevelsel == null || datalevelsel.length == 0)) {
        System.arraycopy(this.highlevelSelectedCells, 0, this.highlevelPreSelectedCells, 0,
                         this.highlevelSelectionCount);
        this.highlevelPreSelectionCount = this.highlevelSelectionCount;
        this.highlevelSelectionCount = grouplevelsel.length;
        System.arraycopy(grouplevelsel, 0, this.highlevelSelectedCells, 0, this.highlevelSelectionCount);
        //Translate this high-level selection to low-level seleciton
        this.translateSelectionHighToLow();
      }else if (datalevelsel != null && datalevelsel.length > 0) {
        System.arraycopy(this.lowlevelSelectedCells, 0, this.lowlevelPreSelectedCells, 0,
                         this.lowlevelSelectionCount);
        this.lowlevelPreSelectionCount = this.lowlevelSelectionCount;
        this.lowlevelSelectionCount = datalevelsel.length;
        System.arraycopy(datalevelsel, 0, this.lowlevelSelectedCells, 0, this.lowlevelSelectionCount);
        //Translate this high-level selection to low-level seleciton
        this.translateSelectionLowToHigh();
      }
    }
  }

  /***
   * Adjust the vector values for a cell is only part of its data items are selected.
   */
  private double[][] originalnewmeans;
  private void adjustSyntheticValues(){
    originalnewmeans = new double[newmeanvalues.length][newmeanvalues[0].length];
    for (int i = 0; i < this.newmeanvalues.length; i++) {
      double value;
      double span;
      double middle;
      int v, position;
      for (int d = 0; d < this.newmeanvalues[i].length; d++) {
        originalnewmeans[i][d] = newmeanvalues[i][d];
        int numIntervals = this.nestedMeans[d].length - 1;
        value = this.newmeanvalues[i][d];
        for (v = 1; v < numIntervals; v++) {
          if (value < this.nestedMeans[d][v])
            break;
        }

        position = v - 1;
        span = this.nestedMeans[d][position + 1] - this.nestedMeans[d][position];
        middle = (this.nestedMeans[d][position + 1] +
                  this.nestedMeans[d][position]) / 2;
        if (Double.isNaN(span) || Double.isNaN(middle) ||
            Double.compare(span, 0) == 0)
          this.newmeanvalues[i][d] = position;
        else
          this.newmeanvalues[i][d] = ( (value - middle) / span) + position;
      }
    }
  }


  /***
   * This method is only called right after explicitly calling the selectionChanged() method.
   */
  protected double[] getSelectedPortions(){
    return this.portions;
  }

  /***
   * This method is only called right after explicitly calling the selectionChanged() method.
   */
  protected int[] getHighLevelSelection(){
    return this.highlevelSelectedCells;
  }


  /***
   * When density threshold changed, this viewer needs to get the recent dense
   * cells and draw them.
   **/
/*  public void DensityThresholdChanged(HighDimensionalClusteringEvent e) {
    this.repaint();
  }

  public void ClusteringChanged(HighDimensionalClusteringEvent e) {
    this.repaint();
  }*/

  private transient Vector selectionListeners;
  public synchronized void addSelectionListener(SelectionListener l) {
    Vector v = selectionListeners == null ? new Vector(2) :
        (Vector) selectionListeners.clone();
    if (!v.contains(l)) {
      v.addElement(l);
      selectionListeners = v;
    }
  }

  public synchronized void removeSelectionListener(SelectionListener l) {
    if (selectionListeners != null && selectionListeners.contains(l)) {
      Vector v = (Vector) selectionListeners.clone();
      v.removeElement(l);
      selectionListeners = v;
    }
  }
  protected void fireSelectionChanged(SelectionEvent e) {
    this.portions = null;
    if (selectionListeners != null) {
      Vector listeners = selectionListeners;
      int count = listeners.size();
      for (int i = 0; i < count; i++) {
        ( (SelectionListener) listeners.elementAt(i)).selectionChanged(e);
      }
    }
  }

  private transient Vector colorArrayListeners;
  public synchronized void removeColorArrayListener(ColorArrayListener l) {
    if (colorArrayListeners != null && colorArrayListeners.contains(l)) {
      Vector v = (Vector) colorArrayListeners.clone();
      v.removeElement(l);
      colorArrayListeners = v;
    }
  }

  public synchronized void addColorArrayListener(ColorArrayListener l) {
    Vector v = colorArrayListeners == null ? new Vector(2) :
        (Vector) colorArrayListeners.clone();
    if (!v.contains(l)) {
      v.addElement(l);
      colorArrayListeners = v;
    }
  }

  protected void fireColorArrayChanged(ColorArrayEvent e) {
    if (colorArrayListeners != null) {
      Vector listeners = colorArrayListeners;
      int count = listeners.size();
      for (int i = 0; i < count; i++) {
        ( (ColorArrayListener) listeners.elementAt(i)).colorArrayChanged(e);
      }
    }
  }

  private double[] yCellMax;
  private double[] yCellMin;
  private double[] yDataMax; //not cell max
  private double[] yDataMin;
  private double[] ySyntheticMax;
  private double[] ySyntheticMin;
}
