/* begin adoption from GRASS r.buffer module */

#include <math.h>
#include "buffer.h"
#include "distance.h"
#include "local_proto.h"


#define PI	3.14159265358979323846
#define Radians(x) ((x) * PI/180.0)
#define Degrees(x) ((x) * 180.0/PI)


static int cur_row;
static double boa;
static double f;
static double ff64;
static double al;

static double t1,t2,t3,t4,t1r,t2r;

double G_geodesic_distance_lon_to_lon (double lon1,double lon2)
{
    double a, cd, d, e, /*dl,*/
	   q, sd, sdlmr, 
	   t, u, v, x, y;


    sdlmr = sin(Radians(lon2-lon1)/2);

    /* special case - shapiro */
    if (sdlmr == 0.0 && t1r == t2r) return 0.0;

    q = t3+sdlmr*sdlmr*t4;

    /* special case - shapiro */
    if (q == 1.0) return PI *al;

/* Mod: shapiro
 * cd=1-2q is ill-conditioned if q is small O(10**-23)
 *   (for high lats? with lon1-lon2 < .25 degrees?)
 *   the computation of cd = 1-2*q will give cd==1.0.
 * However, note that t=dl/sd is dl/sin(dl) which approaches 1 as dl->0.
 * So the first step is to compute a good value for sd without using sin()
 *   and then check cd && q to see if we got cd==1.0 when we shouldn't.
 * Note that dl isn't used except to get t,
 *   but both cd and sd are used later
 */

/* original code
    cd=1-2*q;
    dl=acos(cd);
    sd=sin(dl);
    t=dl/sd;
*/

    cd = 1-2*q;                 /* ill-conditioned subtraction for small q */
/* mod starts here */
    sd = 2* sqrt (q - q * q);	/* sd^2 = 1 - cd^2 */
    if (q != 0.0 && cd == 1.0)  /* test for small q */
	t = 1.0;
    else if (sd == 0.0)
	t = 1.0;
    else
	t = acos(cd)/sd;          /* don't know how to fix acos(1-2*q) yet */
/* mod ends here */

    u = t1/(1-q);
    v = t2/q;
    d = 4*t*t;
    x = u+v;
    e = -2*cd;
    y = u-v;
    a = -d*e;

    return ( al * sd *
		( t - f/4 * (t*x-y) +
		  ff64 *
		  (
		    x * ( a + (t - (a+e)/2) * x) +
		    y* (-2 * d + e*y)
		    + d*x*y
		  )
		)
	    );
}

int begin_distance (int row)
{
    cur_row = row;
    return 0;
}

    /* Determine number of columns for each distance zone
     *    (-1 means doesn't occur)
     * Returns: first zone to occur (-1 if none)
     * Note: modifies distances structure (global)
     */

int find_distances (int row)
{
    int i;
    double dist;
    double scale;
    double ns_dist;

    if (row == cur_row)
    {
		if (window.proj == PROJECTION_LL)
		{
			scale = 1.0/G_geodesic_distance_lon_to_lon (0.0, window.ew_res);
			for (i = 0; i < ndist; i++)
			{
			distances[i].prev_ncols = (int) (scale * distances[i].dist);
			distances[i].ncols = find_ll_distance_ncols(i);
			}
		}
		else /* note - see parse_dist.c for more info  */
		{
			for (i = 0; i < ndist; i++)
			distances[i].ncols = distances[i].prev_ncols 
				= (int) distances[i].dist;
		}
    }
    else
    {
		if (window.proj == PROJECTION_LL)
			for (i = 0; i < ndist; i++)
			distances[i].ncols = find_ll_distance_ncols(i);
		else
		{
			i = cur_row - row;
			ns_dist = (i*i) * ns_to_ew_squared;
			for (i = 0; i < ndist; i++)
			{
			dist = distances[i].dist - ns_dist;
			if (dist < 0.0)
				distances[i].ncols = -1;
			else
				distances[i].ncols = (int) dist;
			}
		}
    }

    for (i = 0; i < ndist; i++)
	if (distances[i].ncols >= 0)
	    return i;
    return -1;
}

int reset_distances (void)
{
    register int i;

    for (i = 0; i < ndist; i++)
	distances[i].ncols = distances[i].prev_ncols ;

    return 0;
}

int find_ll_distance_ncols (int i)
{
    int col;
    double lon;
    register double d;
    register double dist;

	/* use the ncols from previous distances as a starting point
	 * to figure the new ncols
	 */

    col = distances[i].ncols-1;
    if (col < 0) col = 0;
    dist = distances[i].dist;

    lon = window.ew_res * col;

    d = G_geodesic_distance_lon_to_lon (0.0, lon);
    if (d > dist) /*backup */
    {
		while (d > dist)
		{
			if(--col < 0) break;
			lon -= window.ew_res;
			d = G_geodesic_distance_lon_to_lon (0.0, lon);
		}
	return col;
    }
    if (d == dist)
	return col;

    while (d < dist && lon < 180.00 && col <= window.cols)
    {
	col++;
	lon += window.ew_res;
	d = G_geodesic_distance_lon_to_lon (0.0, lon);
    }
    return col-1;
}

/* end adoption from GRASS r.buffer module */
